from pyexperian.lib import dicttoxml
from pyexperian import constants, exceptions
from xml.dom.minidom import parseString
import requests
import urllib
import re
import time
import logging


def enable_debug(filename='pyexperian.log'):
    import datetime
    print('Debug mode is on. Events are logged at: %s' % filename)
    logging.basicConfig(filename=filename, level=logging.INFO)
    logging.info('\nLogging session starts: %s' % (str(datetime.datetime.today())))


def disable_debug():
    logging.basicConfig(level=logging.WARNING)
    print('Debug mode is off.')


class BaseProduct():
    failed_auth_attempts = 0

    def __init__(self, config, ecals):
        self.config = config
        self.ecals = ecals

    # Wrap the product data with request headers.
    def _wrap_with_header(self, product_data={}):
        return {
            'EAI': self.config['eai'],
            'DBHost': self.config['db_host'],
            'ReferenceId': self.config.get('reference_id', ''),
            'Request xmlns="http://www.experian.com/WebDelivery" version="1.0"': {
                'Products': product_data
            }
        }

    def _get_base_request_data(self):
        return {
            'Subscriber': {
                'OpInitials': self.config.get('op_initials', 'OP'),
                'SubCode': self.config['sub_code']
            },
            'OutputType': {
                'XML': {
                    'Verbose': 'Y'
                }
            },
            'Vendor': {
                'VendorNumber': self.config['vendor_number']
            }
        }


    # Translates value to 'Y' or 'N'
    @classmethod
    def _translate_to_bool(cls, val):
        if type(val) == str:
            val = val.upper()
            if val in ['Y', 'N']:
                return val

        return 'Y' if bool(val) else 'N'

    @classmethod
    def _translate_addons(cls, addons_data={}):
        addons = {}

        # Include intelliscore
        if addons_data.get('score', None):
            addons['SCORE'] = cls._translate_to_bool(addons_data['score'])

        # Request owner profile only (BOP)
        if addons_data.get('stand_alone', None):
            addons['StandAlone'] = cls._translate_to_bool(addons_data['stand_alone'])

        # Request Business Profile with Intelliscore
        if addons_data.get('bp', None):
            addons['BP'] = cls._translate_to_bool(addons_data['bp'])

        # Return a list of similars
        if addons_data.get('list', None):
            addons['List'] = cls._translate_to_bool(addons_data['list'])

        return addons

    @classmethod
    def _translate_address(cls, address_data={}):
        address = {}

        if address_data.get('street', None):
            address['Street'] = address_data['street']
        elif address_data.get('street1', None):
            address['Street'] = "%s %s" % (address_data['street1'], address_data.get('street2', ''))

        if address_data.get('city', None):
            address['City'] = address_data['city']

        if address_data.get('state', None):
            address['State'] = address_data['state']

        if address_data.get('zip', None):
            address['Zip'] = address_data['zip']

        return address

    @classmethod
    def _translate_business(cls, business_data={}):
        business = {}
        has_bin = False

        if business_data.get('experian_bin', None):
            has_bin = True
            business['BISFileNumber'] = business_data['experian_bin']

        if business_data.get('bis_file_number', None):
            has_bin = True
            business['BISFileNumber'] = business_data['bis_file_number']

        if business_data.get('name', None):
            business['BusinessName'] = business_data['name']
        elif not has_bin:
            raise exceptions.IncompleteBusinessException('Name is required.')

        if business_data.get('alt_name', None):
            business['AlternateName'] = business_data['alt_name']

        if business_data.get('tax_id', None):
            business['TaxId'] = business_data['tax_id']

        if business_data.get('phone', None):
            business['Phone'] = {'Number': re.sub(r'\D', '', business_data['phone'])}

        if business_data.get('address', None):
            address = cls._translate_address(business_data['address'])
            if not ('City' in address and 'State' in address and 'Zip' in address):
                raise exceptions.IncompleteBusinessException('City, State, and Zip are required.')

            business['CurrentAddress'] = address
        elif not has_bin:
            raise exceptions.IncompleteBusinessException('Address is required.')

        if business_data.get('bis_list_number', None):
            business['BISListNumber'] = business_data['bis_list_number']

        return business

    @classmethod
    def _translate_owner(cls, owner_data={}):
        business_owner = {}
        owner_name = {}

        if owner_data.get('first_name', None):
            owner_name['First'] = owner_data['first_name']

        if owner_data.get('middle_name', None):
            owner_name['Middle'] = owner_data['middle_name']

        if owner_data.get('last_name', None):
            owner_name['Surname'] = owner_data['last_name']

        if owner_data.get('suffix', None):
            owner_name['Gen'] = owner_data['suffix']

        if owner_name and 'First' in owner_name and 'Surname' in owner_name:
            business_owner['OwnerName'] = owner_name
        else:
            raise exceptions.IncompleteOwnerException('First and last name are required.')

        if owner_data.get('ssn', None):
            business_owner['SSN'] = owner_data['ssn']

        if owner_data.get('age', None):
            business_owner['Age'] = owner_data['age']

        if owner_data.get('dob', None):
            business_owner['DOB'] = owner_data['dob']

        if owner_data.get('yob', None):
            business_owner['YOB'] = owner_data['yob']

        if owner_data.get('title', None):
            business_owner['Title'] = owner_data['title']

        if owner_data.get('address', None):
            address = cls._translate_address(owner_data['address'])

            if not ('Street' in address and 'City' in address and 'State' in address and 'Zip' in address):
                raise exceptions.IncompleteOwnerException('Street, City, State, and Zip are required.')

            business_owner['CurrentAddress'] = address
        else:
            raise exceptions.IncompleteOwnerException('Address is required.')

        driver_license = {}
        if owner_data.get('driver_license_num', None):
            driver_license['Number'] = owner_data['driver_license_num']

        if owner_data.get('driver_license_state', None):
            driver_license['State'] = owner_data['driver_license_state']

        if driver_license:
            business_owner['DriverLicense'] = driver_license

        if owner_data.get('age', None):
            business_owner['Age'] = owner_data['age']

        return business_owner

    # Finalizes request data and converts to XML string.
    @classmethod
    def _to_xml(cls, data_dict={}):
        return dicttoxml.dicttoxml(data_dict, attr_type=False, custom_root='NetConnectRequest')

    @classmethod
    def _log_pretty_xml(cls, xml, _header=None):
        header = "\n======%s======\n" % _header.upper() if _header else "\n"
        logging.info("%s%s" % (header, parseString(xml).toprettyxml()))

    def _post_xml(self, xml):
        # Lock them out if too many bad auth attempts
        if BaseProduct.failed_auth_attempts >= constants.MAX_AUTH_ATTEMPTS:
            raise exceptions.MaxAuthAttemptsException()

        url = self.ecals.get_net_connect_url()

        self._log_pretty_xml(xml, 'request')

        logging.info("Net Connect URL: %s" % url)

        data = "NETCONNECT_TRANSACTION=%s" % (urllib.quote_plus(xml))
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        auth = (self.config['user_id'], self.config['user_pw'])

        response = requests.post(url, headers=headers, auth=auth, data=data)

        logging.info(response.text)

        if re.search('^<\?xml', response.text, re.IGNORECASE):
            self._log_pretty_xml(response.text, 'response')

            BaseProduct.failed_auth_attempts = 0

        elif re.search('^<(!DOCTYPE )?html', response.text, re.IGNORECASE):

            # TODO using DEMO environment this is the only way to test for bad AUTH.
            if re.search('app\.logonUrl', response.text):
                BaseProduct.failed_auth_attempts += 1
                raise exceptions.FailedAuthException()

        return response.text


class BusinessPremierProfile(BaseProduct):
    product_id = constants.BUSINESS_PREMIER_PROFILE_ID

    def query(self, business={}, owner={}, addons={}):
        request_data = self._get_base_request_data()

        defaults = {}

        request_data.update(defaults)

        if business:
            request_data['BusinessApplicant'] = self._translate_business(business)
        else:
            raise exceptions.IncompleteBusinessException('Business data is required.')

        if addons:
            request_data['AddOns'] = self._translate_addons(addons)

        xml = self._to_xml(
            self._wrap_with_header({self.product_id: request_data}))

        return self._post_xml(xml)


class BusinessOwnerProfile(BaseProduct):
    product_id = constants.BUSINESS_OWNER_PROFILE_ID

    def query(self, business={}, owner={}, addons={}):
        request_data = self._get_base_request_data()

        defaults = {
            'AddOns': {
                'StandAlone': 'Y'
            }
        }

        if owner:
            # TODO
            defaults['Options'] = {'CustomerName': 'CustomerName'}

        request_data.update(defaults)

        if owner:
            request_data['BusinessOwner'] = self._translate_owner(owner)
        else:
            raise exceptions.IncompleteOwnerException('Owner data is required.')

        if business:
            request_data['BusinessApplicant'] = self._translate_business(business)
        else:
            raise exceptions.IncompleteBusinessException('Business data is required.')

        if addons:
            request_data['AddOns'] = self._translate_addons(addons)

        xml = self._to_xml(
            self._wrap_with_header({self.product_id: request_data}))

        return self._post_xml(xml)


class SBCS(BaseProduct):
    product_id = constants.SBCS_ID

    def query(self, business={}, addons={}, owner={}):
        request_data = self._get_base_request_data()

        defaults = {
            'AddOns': {
                'SCORE': 'Y',
                'BP': 'Y'
            }
        }

        request_data.update(defaults)

        if business:
            request_data.update({
                'BusinessApplicant': self._translate_business(business)
            })

        if addons:
            request_data.update({
                'AddOns': self._translate_addons(addons)
            })

        xml = self._to_xml(
            self._wrap_with_header({self.product_id: request_data}))

        return self._post_xml(xml)


# Pass in the entire NetConnectRequest XML
class Raw(BaseProduct):

    def query(self, xml):
        return self._post_xml(xml.strip())


class Ecals():

    def __init__(self, ecals_url):
        self.ecals_url = ecals_url
        self.net_connect_url = (None, None)  # (url, time_pulled)

    def get_net_connect_url(self):
        # Check if Net Connect URL is expired.
        last_time_pulled = self.net_connect_url[1]
        if not last_time_pulled or ((time.time() - last_time_pulled) >= constants.URL_EXPIRATION_IN_SECONDS):
            logging.info("Net Connect URL is absent or expired.")
            self.net_connect_url = (self._fetch_net_connect_url(), time.time())

        return self.net_connect_url[0]

    def _fetch_net_connect_url(self):
        logging.info("Fetching new Net Connect URL from ECALS.")
        response = requests.get(self.ecals_url)
        if response.status_code == constants.HTTP_STATUS_OK:
            net_connect_url = response.text
            if Ecals.is_valid_net_connect_url(net_connect_url):
                return net_connect_url
            else:
                raise exceptions.InvalidNetConnectUrlException()
        else:
            raise exceptions.EcalsLookupException()

    @classmethod
    def is_valid_net_connect_url(cls, url):
        hostname = Ecals.get_hostname_from_url(url)
        return '.experian.com' in hostname

    # https://www.example.com/blog -> www.example.com
    @classmethod
    def get_hostname_from_url(cls, url):
        match = re.search(r'^https?://([^/]+).*$', url)
        return match.group(1) if match else None




# These tests verify the Experian authentication exam.

from pyexperian import services, exceptions
from nose.tools import raises, with_setup
import requests
import re


config = {}


def setup():
    # These values don't matter since exceptions are thrown before we get passed SSL
    config['user_id'] = 'USER_ID'
    config['user_pw'] = 'USER_PW'
    config['eai'] = 'EAI'
    config['vendor_number'] = 'VENDOR_NUMBER'
    config['sub_code'] = 'SUB_CODE'
    config['db_host'] = 'DB_HOST'
    config['test_subcode'] ='TEST_SUBCODE'


def teardown():
    for key in config.items():
        config[key] = None


@with_setup(setup, teardown)
@raises(exceptions.InvalidNetConnectUrlException)
def test_auth_case_1():
    _ecals = services.Ecals("http://www.experian.com/lookupServlet1?lookupServiceName=AccessPoint&lookupServiceVersion=1.0&serviceName=NetConnect&serviceVersion=0.1&responseType=text/plain")
    _ecals.get_net_connect_url()

@with_setup(setup, teardown)
def test_auth_case_2():
    ecals = services.Ecals('http://www.experian.com/lookupServlet1?lookupServiceName=AccessPoint&lookupServiceVersion=1.0&serviceName=NetConnect&serviceVersion=0.2&responseType=text/plain')
    bpp = services.BusinessPremierProfile(config, ecals);

    try:
        bpp.query(business={'name': 'norecordco', 'address':{'street': '123 main street', 'city': 'buena park', 'state': 'CA', 'zip': '90620'}})
    except requests.exceptions.SSLError as e:
        assert re.search("hostname .* doesn't match", str(e)) is not None


@with_setup(setup, teardown)
def test_auth_case_3():
    ecals = services.Ecals('http://www.experian.com/lookupServlet1?lookupServiceName=AccessPoint&lookupServiceVersion=1.0&serviceName=NetConnect&serviceVersion=0.3&responseType=text/plain')
    bpp = services.BusinessPremierProfile(config, ecals)

    try:
        bpp.query(business={'name': 'norecordco', 'address': {'street': '123 main street', 'city': 'buena park', 'state': 'CA', 'zip': '90620'}})
    except requests.exceptions.SSLError as e:
        assert re.search('certificate verify failed', str(e)) is not None


@with_setup(setup, teardown)
@raises(requests.exceptions.ConnectionError)
def test_auth_case_4():
    ecals = services.Ecals('http://www.experian.com/lookupServlet1?lookupServiceName=AccessPoint&lookupServiceVersion=1.0&serviceName=NetConnect&serviceVersion=0.4&responseType=text/plain')
    bpp = services.BusinessPremierProfile(config, ecals)

    bpp.query(business={'name': 'norecordco', 'address': {'street': '123 main street', 'city': 'buena park', 'state': 'CA', 'zip': '90620'}})


from pyexperian import services


def test_address_street():
    address = {'city': 'austin', 'state': 'tx', 'zip': '78701', 'street1': 'first part', 'street2': 'second part'}
    translated = services.BaseProduct._translate_address(address)
    assert translated['Street'] == ("%s %s" % (address['street1'], address['street2']))

    address = {'city': 'austin', 'state': 'tx', 'zip': '78701', 'street1': 'first part', 'street2': 'second part', 'street': 'priority'}
    translated = services.BaseProduct._translate_address(address)
    assert translated['Street'] == address['street']

def test_good_business():
    business = {'name': 'Franklin BBQ', 'address': {'city': 'austin', 'state': 'tx', 'zip': '78701', 'street': ''}}
    translated = services.BaseProduct._translate_business(business)
    assert 'BusinessName' in translated
    assert 'CurrentAddress' in translated
    assert 'Street' not in translated['CurrentAddress'] and 'City' in translated['CurrentAddress'] and 'State' in translated['CurrentAddress'] and 'Zip' in translated['CurrentAddress']

def test_bools():
    assert services.BaseProduct._translate_to_bool(True) == 'Y'
    assert services.BaseProduct._translate_to_bool(False) == 'N'
    assert services.BaseProduct._translate_to_bool('y') == 'Y'
    assert services.BaseProduct._translate_to_bool('Y') == 'Y'
    assert services.BaseProduct._translate_to_bool('n') == 'N'
    assert services.BaseProduct._translate_to_bool('true') == 'Y'
    assert services.BaseProduct._translate_to_bool('false') == 'Y'



from pyexperian.lib import dicttoxml


def _uglify_xml(xml_output):
    ugly = xml_output.replace("    ", "").replace("\n", "")
    return ugly


def test_xml_request_building():
    d = {
        'EAI': '11111111',
        'DBHost': 'BISPROD',
        'ReferenceId': 'user1abc001',
        'Request xmlns="http://www.experian.com/WebDelivery" version="1.0"': {
            'Products': {
                'PremierProfile': {
                    'Subscriber': {
                        'OpInitials': 'GG',
                        'SubCode': '0123456'
                    },
                    'OutputType': {
                        'XML': {
                            'Verbose': 'Y'
                        }
                    }
                }
            }
        }
    }

    xml_out = dicttoxml.dicttoxml(d, attr_type=False, custom_root='NetConnectRequest xmlns="http://www.experian.com/NetConnect" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.experian.com/NetConnect NetConnect.xsd"')

    assert xml_out == _uglify_xml("""
<?xml version="1.0" encoding="UTF-8" ?>
<NetConnectRequest xmlns="http://www.experian.com/NetConnect" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.experian.com/NetConnect NetConnect.xsd">
    <EAI>11111111</EAI>
    <DBHost>BISPROD</DBHost>
    <ReferenceId>user1abc001</ReferenceId>
    <Request xmlns="http://www.experian.com/WebDelivery" version="1.0">
        <Products>
            <PremierProfile>
                <Subscriber>
                    <SubCode>0123456</SubCode>
                    <OpInitials>GG</OpInitials>
                </Subscriber>
                <OutputType>
                    <XML>
                        <Verbose>Y</Verbose>
                    </XML>
                </OutputType>
            </PremierProfile>
        </Products>
    </Request>
</NetConnectRequest>""")




import xmltodict
from pyexperian import constants, exceptions

class BaseParser():

    # Helper function to simplify looking for a value nested deeply within a dictionary
    @classmethod
    def _get_dict_value(cls, d, key_list):
        if type(d) not in [dict, list, xmltodict.OrderedDict]:
            return d

        if type(key_list) is not list:
            key_list = [key_list]

        for key in key_list:
            if type(key) == str and key in d:
                d = d[key]
            elif type(key) == int and len(d) >= key:
                d = d[key]
            else:
                return None

        return d

    def has_list(self):
        return 'ListOfSimilars' in self.dict[self.product_id]

    def business_found(self):
        profile_type_code = self._get_dict_value(self.dict, [self.product_id, 'BusinessNameAndAddress', 'ProfileType', '@code'])
        return not self.has_list() and (not profile_type_code or profile_type_code.strip() != 'NO RECORD')

    def __init__(self, xml):
        self.xml = xml

        d = xmltodict.parse(xml)
        self.dict = d['NetConnectResponse']

        # Check for 'Bad Request' error
        error_message = self._get_dict_value(self.dict, ['ErrorMessage'])
        if error_message:
            error_tag = self._get_dict_value(self.dict, ['ErrorTag'])
            raise exceptions.BadRequestException('Message: %s, Tag: %s' % (error_message, error_tag))

        self.dict = self.dict['Products']


class BusinessOwnerProfile(BaseParser):
    product_id = constants.BUSINESS_OWNER_PROFILE_ID

    def __init__(self, xml):
        BaseParser.__init__(self, xml)

        if self._get_dict_value(self.dict, [self.product_id, 'ProcessingMessage', 'ProcessingAction', '@code']) == constants.TERMS_RESPONSE_CODE:
            raise exceptions.TermsException()

    def owner_found(self):
        return None is not self._get_dict_value(self.dict, ['CreditProfile', 'ProfileSummary'])


class BusinessPremierProfile(BaseParser):
    product_id = constants.BUSINESS_PREMIER_PROFILE_ID

    def _similar_to_dict(self, similar):
        return {
            'experian_bin': similar['ExperianFileNumber'],
            'name': similar['BusinessName'],
            'address': {
                'city': similar['City'],
                'state': similar['State'],
                'zip': similar['Zip'],
                'street': similar['StreetAddress']
            }
        }

    def get_business(self):
        business_info = self._get_dict_value(self.dict, [self.product_id, 'ExpandedBusinessNameAndAddress'])
        if business_info:
            return {
                'name': business_info['BusinessName'],
                'experian_bin': business_info['ExperianBIN'],
                'phone': business_info['PhoneNumber'],
                'address': {
                    'city': business_info['City'],
                    'state': business_info['State'],
                    'zip': business_info['Zip'],
                    'street': business_info['StreetAddress']
                }
            }

        return None


    def get_list(self):
        similars = self._get_dict_value(self.dict, [self.product_id, 'ListOfSimilars'])

        if type(similars) is not list:
            similars = [similars]

        return map(self._similar_to_dict, similars)


class SBCS(BaseParser):
    product_id = constants.SBCS_ID




import os
import re

v = open(os.path.join(os.path.dirname(__file__), 'pyexperian', '__init__.py'))
VERSION = re.compile(r".*__version__ = '(.*?)'", re.S).match(v.read()).group(1)
v.close()

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

config = {
    'description': 'Python wrapper for Experian\'s Net Connect API',
    'author': 'Carlos Eduardo Rivera',
    'url': 'https://github.com/AbleEng/pyexperian',
    'download_url': 'https://github.com/AbleEng/pyexperian',
    'author_email': 'carlos@hiable.com',
    'version': VERSION,
    'install_requires': ['requests>=2.2.1', 'xmltodict>=0.9.0'],
    'tests_require': ['nose>=1.3.4'],
    'packages': ['pyexperian', 'pyexperian.test', 'pyexperian.lib'],
    'scripts': [],
    'name': 'pyexperian',
    'keywords': 'experian netconnect ecals'
}

setup(**config)





class InvalidNetConnectUrlException(Exception):
    def __init__(self):
        super(InvalidNetConnectUrlException, self).__init__("Net Connect URL is invalid.")


class EcalsLookupException(Exception):
    def __init__(self):
        super(EcalsLookupException, self).__init__("Failed to retrieve Net Connect URL via ECALS lookup.")


class MaxAuthAttemptsException(Exception):
    def __init__(self):
        super(MaxAuthAttemptsException, self).__init__("You've maxed out your authentication attempts.")


class FailedAuthException(Exception):
    def __init__(self):
        super(FailedAuthException, self).__init__("Bad credentials.  Did you remember to reset your password?")


class BadRequestException(Exception):
    def __init__(self, msg=''):
        super(BadRequestException, self).__init__(msg or "Check your config and query data for bad or missing values.")


class TermsException(Exception):
    def __init__(self):
        super(TermsException, self).__init__("Re-submit your request if you have consent to request the customer's personal credit.")


class IncompleteOwnerException(Exception):
    def __init__(self, msg):
        super(IncompleteOwnerException, self).__init__(msg)


class IncompleteBusinessException(Exception):
    def __init__(self, msg):
        super(IncompleteBusinessException, self).__init__(msg)


import xmltodict
from pyexperian import services, constants, parsers

xml_data = """
<CATALOG>
    <CD>
        <TITLE>Empire Burlesque</TITLE>
        <ARTIST>Bob Dylan</ARTIST>
        <COUNTRY>USA</COUNTRY>
        <COMPANY>Columbia</COMPANY>
        <PRICE>10.90</PRICE>
        <YEAR>1985</YEAR>
    </CD>
    <CD>
        <TITLE>Hide your heart</TITLE>
        <ARTIST>Bonnie Tyler</ARTIST>
        <COUNTRY>UK</COUNTRY>
        <COMPANY>CBS Records</COMPANY>
        <PRICE>9.90</PRICE>
        <YEAR>1988</YEAR>
    </CD>
    <CD>
        <TITLE>Greatest Hits</TITLE>
        <ARTIST>Dolly Parton</ARTIST>
        <COUNTRY>USA</COUNTRY>
        <COMPANY>RCA</COMPANY>
        <PRICE>9.90</PRICE>
        <YEAR>1982</YEAR>
    </CD>
    <CD>
        <TITLE>Still got the blues</TITLE>
        <ARTIST>Gary Moore</ARTIST>
        <COUNTRY>UK</COUNTRY>
        <COMPANY>Virgin records</COMPANY>
        <PRICE>10.20</PRICE>
        <YEAR>1990</YEAR>
    </CD>
</CATALOG>
"""


def test_xmltodict_parsing():
    data = xmltodict.parse(xml_data)

    assert data['CATALOG']['CD'][0]['PRICE'] == '10.90'

    cds = data['CATALOG']['CD']

    assert len(cds) == 4
    assert cds[3]['YEAR'] == '1990'


def test_get_val_helper():
    data = xmltodict.parse(xml_data)

    assert data['CATALOG']['CD'][0]['PRICE'] == parsers.BaseParser._get_dict_value(data, ['CATALOG', 'CD', 0, 'PRICE'])

    assert parsers.BaseParser._get_dict_value(data, ['CATALOG', 'CARLOS']) is None


def test_parse_error_code():
    data = xmltodict.parse("""<?xml version="1.0" standalone="no"?><NetConnectResponse xmlns="http://www.experian.com/NetConnectResponse"><CompletionCode>0000</CompletionCode><ReferenceId>StandAlone BOP</ReferenceId><TransactionId>14996435</TransactionId><Products xmlns="http://www.experian.com/ARFResponse"><BusinessProfile><ProcessingMessage><ProcessingAction code="031">BUSINESS OWNER/SMALL BUSINESS INTELLISCORE TERMS MESSAGE</ProcessingAction></ProcessingMessage></BusinessProfile></Products></NetConnectResponse>""")

    assert parsers.BaseParser._get_dict_value(data, ['NetConnectResponse', 'Products', 'BusinessProfile', 'ProcessingMessage', 'ProcessingAction', '@code']) == constants.TERMS_RESPONSE_CODE